// Copyright David Abrahams 2002.
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
#ifndef NOT_SPECIFIED_DWA2002321_HPP
# define NOT_SPECIFIED_DWA2002321_HPP

namespace boost { namespace python { namespace detail { 

  struct not_specified {};

}}} // namespace boost::python::detail

#endif // NOT_SPECIFIED_DWA2002321_HPP
